/*
 * $Id: main.c,v 1.5.4.3 2005/08/18 13:23:11 dahms Exp $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gtk/gtkgl.h>

#include <g3d/types.h>

#include "main.h"
#include "model.h"
#include "interface.h"
#include "trackball.h"

static gboolean parse_only = FALSE;

int main_parseargs(int *argc, char ***argv, G3DViewer *viewer);

int main(int argc, char **argv)
{
	G3DViewer *viewer;

	gtk_init(&argc, &argv);
	gtk_gl_init(&argc, &argv);

	viewer = g_new0(G3DViewer, 1);
	viewer->zoom = 45;
	viewer->mouse.beginx = 0;
	viewer->mouse.beginy = 0;
	viewer->filename = NULL;
	viewer->bgcolor[0] = 0.9;
	viewer->bgcolor[1] = 0.8;
	viewer->bgcolor[2] = 0.6;
	viewer->bgcolor[3] = 1.0;
	viewer->glflags =
		/* G3D_FLAG_GL_SPECULAR | */
		G3D_FLAG_GL_SHININESS |
		G3D_FLAG_GL_ALLTWOSIDE |
		G3D_FLAG_GL_TEXTURES;

	viewer->g3dcontext = g3d_context_new();

	main_parseargs(&argc, &argv, viewer);

	trackball(viewer->quat, 0.0, 0.0, 0.0, 0.0);

	interface_init(viewer);

	if(viewer->filename != NULL)
	{
		model_load(viewer);
	}
	else
		interface_showopendialog(viewer);

	if(parse_only)
		return EXIT_SUCCESS;

	interface_update_window(viewer);

	gtk_main();

	g3d_context_free(viewer->g3dcontext);

	gtk_exit(0);

	return EXIT_SUCCESS;
}

void main_showhelp(void)
{
	printf(
		"g3dviewer - a 3D model viewer\n"
		"\n"
		"usage: g3dviewer [--option ...] [<filename>]\n"
		"where option can be:\n"
		"  --help               show this help screen\n"
		);
	exit(1);
}

int main_parseargs(int *argc, char ***argv, G3DViewer *viewer)
{
	/* program name */
	(*argc)--;
	(*argv)++;
	while(*argc > 0)
	{
#if DEBUG > 3
		g_printerr("arg: %s\n", **argv);
#endif
		if(strcmp(**argv, "--help") == 0) main_showhelp();
		else if(strcmp(**argv, "--parse-only") == 0)
		{
			parse_only = TRUE;
		}
		else
		{
			viewer->filename = **argv;
		}
		(*argv)++;
		(*argc)--;
	}

	return EXIT_SUCCESS;
}

